/***********************************************************************
** pmwiki.css
**  Copyright 2004-2017 Patrick R. Michaud (pmichaud@pobox.com)
**  Copyright 2006 Hagan Fox
**  This file is part of PmWiki; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published
**  by the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.  See pmwiki.php for full details.
**  
**  File maintained by Petko Yotov www.pmwiki.org/petko
***********************************************************************/

/* This sets the overall frame for the site */
body {
  margin:0px; background-color:#f7f7f7; color: black;
  font-family:Arial,Helvetica,sans-serif; font-size:11pt;
}

/* These control the fixed-width text elements of the page */
textarea, pre, code { font-size:0.9em; }
pre, code { font-family:'Lucida Console','Andale Mono','Courier New',Courier,monospace; }
pre { line-height:1.2em; }
pre code, code code, pre pre { font-size:100%; }

/* These primarily adjust the size and spacing of heading elements,
** most browsers have atrocious defaults for these. */
h1, h2, h3, h4, h5, h6 { margin-top:1.0em; margin-bottom:0.6em; }
h1, h2, h3, h6 { font-weight:normal; }
h4, h5 { font-weight:bold; }
h1 code, h2 code, h3 code, h4 code { font-size:1em; }
h1 { font-size:1.8em; }
h2 { font-size:1.44em; }
h3 { font-size:1.22em; }
h4 { font-size:1.07em; }
h5 { font-size:1.0em; }
h6 { font-size:1.0em; }

/* The #wikilogo element is the logo from $PageLogoFmt */
#wikilogo { margin-top:4px; padding:6px; border-bottom:1px #cccccc solid; }

/* This controls the rest of the heading (primarily the search box) */
#wikihead { 
  position:absolute; right:10px; top:10px; 
  font-family:Verdana,sans-serif; font-size:85%; 
}
#wikihead input { font-size:85%; } 

/* These are for the left-sidebar. */
#wikileft {
  width:155px; 
  padding:6px; border-right:1px #cccccc solid; 
  line-height:1.33em;
  font-size:9.4pt; font-family:Verdana,sans-serif;
}
#wikileft .vspace { margin-top:1.125em; }
#wikileft a { text-decoration:none; color:black; }
#wikileft a:hover { text-decoration:underline; color:blue; }
#wikileft ul { list-style:none; padding:0px; margin:0px; }
#wikileft li { margin:0px; padding-left: 6px; }
.sidehead {
  margin:0px; padding:4px 2px 2px 2px;
  font-size:11pt; font-weight:bold; font-style:normal; 
}
.sidehead a 
  { color:#505050; font-weight:bold; font-style:normal; }

/* These affect the main content area. */
#wikibody { 
  padding:0px 10px 10px 10px; background-color:white; 
  font-size:11pt; 
}
#wikicmds { 
  float:right; white-space:nowrap; 
  font-family:Verdana,sans-serif; font-size:80%; 
}
#wikicmds ul { list-style:none; margin:0px; padding:0px; }
#wikicmds li { display:inline; margin:0px 5px; }
#wikicmds li a { text-decoration:none; color:black; border:none; }
#wikicmds li a.createlink { display:none; }
#wikicmds li a:hover { text-decoration:underline; color:blue; }
.pagegroup { margin-top:8px; margin-bottom:2px; }
.pagetitle { line-height:1em; margin:0px; font-size:1.6em; font-weight:normal; }
.wikiaction { margin-top:4px; margin-bottom:4px; }
#wikitext { margin-top:12px; line-height:1.33em; }
#wikitext table { font-size:100%; line-height:1.33em; } /* For MSIE 5.5 */

/* These are for the edit form. */
#wikiedit form { margin:0px; width:100%; }
#wikiedit textarea { width:100%; }
.wikimessage { margin-top:4px; margin-bottom:4px; font-style:italic; }

/* These affect the lines at the very bottom. */
#wikifoot { 
  padding-left:178px; padding-bottom:4px; border-top:1px #cccccc solid;
  font-family:Verdana,sans-serif; font-size:80%; 
}

/* table class=simpletable from pmwiki-responsive.css */
table.simpletable {
  border-collapse: collapse;
}
table.simpletable tbody:first-child tr:nth-child(odd) {
  background-color: #eee;
}
table.simpletable thead + tbody tr:nth-child(even) {
  background-color: #eee;
}
table.simpletable th {
  background-color: #ccc;
}
table.simpletable th, table.simpletable td {
  border: 1px solid #888;
}

/* These affect the printed appearance of the web view (not the separate
** print view) of pages.  The sidebar and action links aren't printed. */
@media print {
  body { width:auto; margin:0px; padding:0.5em; }
  #wikihead, #wikileft, #wikicmds, .footnav { display:none; }
  #wikifoot { padding:2px; }
}
