<?php if (!defined('PmWiki')) exit();
/*  Copyright 2019-2026 Petko Yotov www.pmwiki.org/petko
    This file is part of PmWiki; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  See pmwiki.php for full details.

    File kept for backwards compatibility.
    Please include directly individual scripts as needed.
*/

include_once(__DIR__ . '/multipart.php');
include_once(__DIR__ . '/totp.php');
include_once(__DIR__ . '/pmcrypto.php');
include_once(__DIR__ . '/request.php');
